import { __awaiter, __generator } from "tslib";
import { Task, } from './task';
export var AgentStatus;
(function (AgentStatus) {
    AgentStatus["Online"] = "online";
    AgentStatus["Offline"] = "offline";
    AgentStatus["Pause"] = "pause";
    AgentStatus["BreakOut"] = "break_out";
})(AgentStatus || (AgentStatus = {}));
export var ChannelState;
(function (ChannelState) {
    ChannelState["Waiting"] = "waiting";
    ChannelState["Distribute"] = "distribute";
    ChannelState["Offering"] = "offering";
    ChannelState["Answered"] = "answered";
    ChannelState["Active"] = "active";
    ChannelState["Bridged"] = "bridged";
    ChannelState["Hold"] = "hold";
    ChannelState["Missed"] = "missed";
    ChannelState["WrapTime"] = "wrap_time";
    ChannelState["Processing"] = "processing";
    ChannelState["Transfer"] = "transfer";
})(ChannelState || (ChannelState = {}));
export var ChannelType;
(function (ChannelType) {
    ChannelType["Call"] = "call";
    ChannelType["Email"] = "email";
    ChannelType["Chat"] = "chat";
})(ChannelType || (ChannelType = {}));
var Agent = /** @class */ (function () {
    function Agent(client, info) {
        this.client = client;
        this.info = info;
        this.task = new Map();
        this._channel = info.channels[0]; // todo
        this.lastStatusChange = Date.now() - this.info.status_duration * 1000;
    }
    Object.defineProperty(Agent.prototype, "agentId", {
        get: function () {
            return this.info.agent_id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "onDemand", {
        get: function () {
            return this.info.on_demand;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "status", {
        get: function () {
            return this.info.status;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "statusPayload", {
        get: function () {
            return this.info.status_payload;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "isAdmin", {
        get: function () {
            return this.info.is_admin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "isSupervisor", {
        get: function () {
            return this.info.is_supervisor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "team", {
        get: function () {
            return this.info.team;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "supervisor", {
        get: function () {
            return this.info.supervisor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "auditor", {
        get: function () {
            return this.info.auditor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "channels", {
        get: function () {
            return [this._channel];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "channel", {
        get: function () {
            return this._channel;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Agent.prototype, "stateDuration", {
        get: function () {
            return Math.round((Date.now() - this.lastStatusChange) / 1000);
        },
        enumerable: true,
        configurable: true
    });
    // todo need refactor
    Agent.prototype.onChannelEvent = function (e) {
        var task;
        switch (e.status) {
            case ChannelState.Distribute:
                var distributeEvent = e;
                if (!distributeEvent) {
                    throw new Error('bad event');
                }
                task = new Task(this.client, e, distributeEvent.distribute);
                this.task.set(task.id, task);
                if (task.agentChannelId) {
                    switch (task.channel) {
                        case ChannelType.Call:
                            var call = this.client.callById(task.agentChannelId);
                            if (call && !call.task) {
                                call.task = task;
                            }
                            break;
                        case ChannelType.Chat:
                            var chat = this.client.conversationById(task.agentChannelId);
                            if (chat && !chat.task) {
                                chat.task = task;
                            }
                            break;
                        default:
                    }
                }
                break;
            case ChannelState.Offering:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.offeringAt = e.timestamp;
                }
                break;
            case ChannelState.Bridged:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.bridgedAt = e.timestamp;
                }
                break;
            case ChannelState.Transfer:
                var transfer = e;
                task = this.task.get(transfer.to_attempt_id);
                if (task) {
                    this.task.delete(task.id);
                    task.id = transfer.attempt_id;
                    task.setTransferred(transfer.distribute);
                    this.task.set(task.id, task);
                }
                break;
            case ChannelState.Answered:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.answeredAt = e.timestamp;
                }
                break;
            case ChannelState.Missed:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        var missedEvent = e;
                        if (!missedEvent) {
                            throw new Error('bad event');
                        }
                        this.setChannelStateTimeout(e.channel, e, missedEvent.missed.timeout);
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                        return task;
                    }
                }
                break;
            case ChannelState.WrapTime:
                if (e.attempt_id) {
                    var wrapTimeEvent = e;
                    if (!wrapTimeEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    this.task.delete(e.attempt_id);
                    this.client.reportingChannelTask(task);
                }
                break;
            case ChannelState.Processing:
                if (e.attempt_id) {
                    var processingEvent = e;
                    if (!processingEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.state = e.status;
                        task.closedAt = e.timestamp; // todo
                        task.setProcessing(processingEvent.processing);
                        this.setChannelStateTimeout(e.channel, e, processingEvent.processing.timeout);
                        return task;
                    }
                }
                break;
            // TODO
            case ChannelState.Waiting:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                    }
                }
                break;
            default:
            // throw new Error("not found task")
        }
        this.setChannelState(e.channel, e);
        if (task) {
            task.state = e.status;
            return task;
        }
        else {
            return undefined;
        }
    };
    Agent.prototype.online = function (channels, onDemand) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.agentSetOnline(this.agentId, channels, onDemand)];
            });
        });
    };
    Agent.prototype.waiting = function (channel) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.agentSetWaiting(this.agentId, channel)];
            });
        });
    };
    Agent.prototype.pause = function (payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.agentSetPause(this.agentId, payload)];
            });
        });
    };
    Agent.prototype.offline = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.agentSetOffline(this.agentId)];
            });
        });
    };
    Agent.prototype.setStatus = function (e) {
        if (e.status === AgentStatus.Online) {
            this.info.on_demand = e.on_demand || false;
            this._channel = e.channels[0];
        }
        this.info.status = e.status;
        this.info.status_payload = e.status_payload ? e.status_payload : null;
        this.lastStatusChange = Date.now();
    };
    Agent.prototype.directMember = function (memberId, communicationId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("cc_member_direct", {
                        agent_id: this.agentId,
                        member_id: memberId,
                        communication_id: communicationId,
                    })];
            });
        });
    };
    Agent.prototype.offlineMembers = function (q, page, perPage) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("cc_fetch_offline_members", {
                        agent_id: this.agentId,
                        page: page,
                        q: q,
                        per_page: perPage,
                    })];
            });
        });
    };
    Agent.prototype.hasTask = function (task) {
        return this.task.has(task.id);
    };
    Agent.prototype.setChannelState = function (name, e) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status;
        this._channel.joined_at = e.timestamp;
        this._channel.timeout = undefined;
    };
    Agent.prototype.setChannelStateTimeout = function (name, e, timeout) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status; // FIXME
        this._channel.timeout = timeout;
        this._channel.joined_at = e.timestamp;
    };
    return Agent;
}());
export { Agent };
//# sourceMappingURL=agent.js.map