const { app, ipcMain, powerMonitor } = require("electron")
const WebitelWindows = require('./windows')
const WebitelTray = require('./module/webitel_tray')

let win = new WebitelWindows()
let tray = {}

app.commandLine.appendSwitch('ignore-certificate-errors', 'true')
app.commandLine.appendSwitch('allow-insecure-localhost', 'true')

app.setAsDefaultProtocolClient("wtel")

const gotTheLock = app.requestSingleInstanceLock()
if (!gotTheLock) {
    app.quit()
    return
} else {
    app.on('second-instance', (event, argv) => {
        win.restoreWindow()
        //todo fixme make-call
        if (process.platform === 'win32' || process.platform === 'linux') {
            win.makeCall(argv.slice(1)[0])
        }

    })
}

app.on('ready', () => {
    win.start()
    createAndSubscribeTray()
    subscribePowerMonitor()
})

ipcMain.on("file-open", (event, args) => {
    win.opendDialog()
})

ipcMain.on("update-tray", (event, args) => {
    tray.updateTray(args)    
})

ipcMain.on("reload-page", async (event, args) => {
       win.workspace.reload()
})

ipcMain.on("remove-call-info", (event, args) => {
    win.removeCall(args)
})

ipcMain.on("call-action", (event, args) => {
    win.callAction(args)
})

ipcMain.on("set-acive-call", (event, args) => {
    win.setActiveCall(args)
})

ipcMain.on("update-acive-call", (event, args) => {
    win.callNotification.updateActiveCall(args)
})

ipcMain.on("destroy-notification-window", (event, args) => {
    win.callNotification.destroyNotification()
    tray.setDefaultTray()
})

ipcMain.on("open-disconnect-popup", (event, args) => {
    win.showDisconnectNotification()
})

ipcMain.on("hide-disconnect-popup", (event, args) => {
    win.hideDisconnectNotification()
})

ipcMain.on("collaps-window", (event, args) => {
    win.callNotification.collapsWindow()   
})

ipcMain.on("open-close-DevTools", (event, args) => {
    win.openDevTools()   
})

ipcMain.on("resize-popap-win", (event, args) => {
    win.callNotification.resize(args)   
})

ipcMain.on("update-procesing", (event, args) => {
    win.updateProcessing(args)   
})

ipcMain.on("set-processing-property", (event, args) => {
    win.setProcessingProperty(args)   
})

ipcMain.on("send-reporting", (event, args) => {
    win.sendReporting()   
})

ipcMain.on("clear-processing", (event, args) => {
    win.clearProcessing()   
})

ipcMain.on("close-success-message", (event, args) => {
    win.closeSuccessMessage()   
})

ipcMain.on("reset-timer", (event, args) => {
    win.resetTime()   
})

ipcMain.on("collaps-load-window", (event, args) => {
    win.collapsWindow()
})

ipcMain.on("expand-load-window", (event, args) => {
    win.expandWindow()   
})

ipcMain.on("close-load-window", (event, args) => {
    win.closeWindow()   
})

ipcMain.handle("sip_checked", (event, args) => {
    return tray.lStorage.getSIP();
})
ipcMain.handle("restart", (event, args) => {
    console.error('restart')
    app.relaunch({ args: process.argv.slice(1).concat(['--relaunch']) })
    app.exit(0)
})


function createAndSubscribeTray() {
    tray = new WebitelTray()
    tray.on('file-open', () => {
        win.opendDialog()
    })
    tray.on('change-status', (status) => {
        win.changeUserStatus(status)
    })
    tray.on('on-change-lang', (id) => {
        win.onChangeLang(id)
    })
    tray.on('show-hide-window', () => {
        win.setWorkspaceVisible()
    })
    tray.on('on-change-SIP', (value) => {
        win.onChangeSIP(value)
    })    
}

function subscribePowerMonitor() {
    powerMonitor.on('resume', () => {
        console.log('The system is going to ready', new Date());
        if (win) {
            win.resume();
        }
    });

    powerMonitor.on('suspend', () => {
        console.log('The system is going to sleep', new Date());
        if (win) {
            win.sleep();
        }
    });

    powerMonitor.on('unlock-screen', () => {
        console.log('The unlock-screen');
        if (win) {
            win.resume();
        }
    });

}